var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import { List, Map } from 'immutable';
import PropTypes from 'prop-types';
import Paths from './Paths';

var SvgSketchCanvas = function (_React$Component) {
  _inherits(SvgSketchCanvas, _React$Component);

  function SvgSketchCanvas(props) {
    _classCallCheck(this, SvgSketchCanvas);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.state = {
      isDrawing: false,
      history: new List(),
      redoStore: new List(),
      currentPaths: new List()
    };

    _this.handleMouseDown = _this.handleMouseDown.bind(_this);
    _this.handleMouseMove = _this.handleMouseMove.bind(_this);
    _this.handleMouseUp = _this.handleMouseUp.bind(_this);
    _this.getCoordinates = _this.getCoordinates.bind(_this);
    _this.exportAsImage = _this.exportAsImage.bind(_this);
    _this.exportSvg = _this.exportSvg.bind(_this);
    _this.clearCanvas = _this.clearCanvas.bind(_this);
    _this.undo = _this.undo.bind(_this);
    _this.redo = _this.redo.bind(_this);
    _this.svgCanvas = null;
    _this.rawCanvas = null;
    return _this;
  }

  SvgSketchCanvas.prototype.componentDidMount = function componentDidMount() {
    document.addEventListener('mouseup', this.handleMouseUp);
  };

  SvgSketchCanvas.prototype.componentWillUnmount = function componentWillUnmount() {
    document.removeEventListener('mouseup', this.handleMouseUp);
  };

  // Converts mouse coordinates to relative coordinate based on the absolute position of svg


  SvgSketchCanvas.prototype.getCoordinates = function getCoordinates(mouseEvent) {
    var boundingArea = this.svgCanvas.getBoundingClientRect();
    return new Map({
      x: mouseEvent.clientX - boundingArea.left,
      y: mouseEvent.clientY - boundingArea.top
    });
  };

  SvgSketchCanvas.prototype.clearCanvas = function clearCanvas() {
    this.setState(function (prevState) {
      return {
        history: prevState.currentPaths,
        currentPaths: new List()
      };
    });
  };

  SvgSketchCanvas.prototype.undo = function undo() {
    if (this.state.currentPaths.isEmpty()) {
      if (this.state.history.isEmpty()) return;

      this.setState(function (prevState) {
        return {
          history: new List(),
          currentPaths: prevState.history
        };
      });

      return;
    }

    this.setState(function (prevState) {
      return {
        redoStore: prevState.redoStore.push(prevState.currentPaths.get(-1)),
        currentPaths: prevState.currentPaths.pop()
      };
    });
  };

  SvgSketchCanvas.prototype.redo = function redo() {
    if (this.state.redoStore.isEmpty()) return;

    this.setState(function (prevState) {
      return {
        redoStore: prevState.redoStore.pop(),
        currentPaths: prevState.currentPaths.push(prevState.redoStore.get(-1))
      };
    });
  };

  SvgSketchCanvas.prototype.handleMouseDown = function handleMouseDown(mouseEvent) {
    if (mouseEvent.button !== 0) return;

    var mousePoint = this.getCoordinates(mouseEvent);

    this.setState(function (prevState) {
      return {
        isDrawing: true,
        currentPaths: prevState.currentPaths.push(new List([mousePoint]))
      };
    });
  };

  // Creates a image from SVG and renders it on canvas, then exports the canvas as image


  SvgSketchCanvas.prototype.exportAsImage = function exportAsImage(imageType) {
    var _this2 = this;

    return new Promise(function (resolve, reject) {
      try {
        var img = document.createElement('img');
        img.src = 'data:image/svg+xml;base64,' + btoa(_this2.svgCanvas.innerHTML);

        img.onload = function () {
          _this2.rawCanvas.getContext('2d').drawImage(img, 0, 0);

          resolve(_this2.rawCanvas.toDataURL('image/' + imageType));
        };
      } catch (e) {
        reject(e);
      }
    });
  };

  SvgSketchCanvas.prototype.exportSvg = function exportSvg() {
    var _this3 = this;

    return new Promise(function (resolve, reject) {
      try {
        resolve(_this3.svgCanvas.innerHTML);
      } catch (e) {
        reject(e);
      }
    });
  };

  SvgSketchCanvas.prototype.handleMouseMove = function handleMouseMove(mouseEvent) {
    if (!this.state.isDrawing) return;

    var mousePoint = this.getCoordinates(mouseEvent);

    this.setState(function (prevState) {
      return {
        currentPaths: prevState.currentPaths.updateIn([prevState.currentPaths.size - 1], function (path) {
          return path.push(mousePoint);
        })
      };
    });
  };

  SvgSketchCanvas.prototype.handleMouseUp = function handleMouseUp(mouseEvent) {
    if (mouseEvent.button !== 0) return;

    this.setState({
      isDrawing: false
    });
  };

  SvgSketchCanvas.prototype.render = function render() {
    var _this4 = this;

    var _props = this.props,
        width = _props.width,
        height = _props.height,
        strokeColor = _props.strokeColor,
        strokeWidth = _props.strokeWidth,
        styles = _props.styles;


    return React.createElement(
      React.Fragment,
      null,
      React.createElement(
        'div',
        {
          role: 'presentation',
          ref: function ref(element) {
            _this4.svgCanvas = element;
          },
          style: _extends({ width: width, height: height }, styles),
          onMouseDown: this.handleMouseDown,
          onMouseMove: this.handleMouseMove
        },
        React.createElement(
          'svg',
          {
            version: '1.1',
            baseProfile: 'full',
            width: width,
            height: height,
            viewBox: '0 0 ' + width + ' ' + height,
            xmlns: 'http://www.w3.org/2000/svg',
            style: {
              strokeWidth: strokeWidth
            }
          },
          React.createElement('rect', { width: '100%', height: '100%', fill: this.props.canvasColor }),
          React.createElement(Paths, { strokeColor: strokeColor, paths: this.state.currentPaths })
        )
      ),
      React.createElement('canvas', {
        style: { display: 'none' },
        width: width,
        height: height,
        ref: function ref(element) {
          _this4.rawCanvas = element;
        }
      })
    );
  };

  return SvgSketchCanvas;
}(React.Component);

SvgSketchCanvas.defaultProps = {
  width: 600,
  height: 400,
  canvasColor: 'white',
  strokeColor: 'black',
  strokeWidth: 4,
  styles: {
    border: '0.0625rem solid #9c9c9c',
    borderRadius: '0.25rem'
  }
};

SvgSketchCanvas.propTypes = process.env.NODE_ENV !== "production" ? {
  width: PropTypes.number,
  height: PropTypes.number,
  strokeColor: PropTypes.string,
  canvasColor: PropTypes.string,
  strokeWidth: PropTypes.number,
  styles: PropTypes.objectOf(PropTypes.string)
} : {};

export default SvgSketchCanvas;